#!/bin/perl
#################################################################################
# Created       : Martin Foster
# Modified      : 09/19/2000
#################################################################################
#
# Database Setup - Script part of Webchat designed to configure MySQL for use 
# Copyright (C) 2000  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : mfoster@redwhite.com
#       Address : Suite 209 - 355 5th Ave NE
#                 Calgary, Alberta
#                 T2R 0K9
#
#################################################################################

use Cwd;							# Current working directory
use DBI;							# Database independent layer

use String::Random;						# Random string generator


#################################################################################
# Data Members
#################################################################################

#####################
# Database Limits

# PUPPETEER
$PuppeteerLogin = 10;						# Login name
$PuppeteerPassword = CryptLenght('SALT', 'Secret');		# Password
$PupeteerName = 30;						# Full name
$PuppeteerEMail = 45;						# E-Mail address

# PUPPET
$PuppetName = 30;						# Handle Name
$PuppetPic = 200;						# Picture link address
$PuppetTextColour = 8;						# Text colour used by character

# REALM
$RealmName = 30;						# Name of realm

# POST
$PostTo = 30;							# Puppet receiving

# WORD_RESTRICTION
$WordSpelling = 10;						# Word Spelling

# PUPPETEER_REGISTRATION
$PuppeteerConfCode = 10;					# Confirmation code

# WEBCHAT_TEXT
$WebchatTextName = 20;						# Identifying name


#####################
# Database Indentifier

$dbserver = 'localhost';					# Server name 
$dbdatam  = 'mysql';						# MySQL database
$dbdataw  = 'webchat';						# Webchat database
$dbuserm  = 'root';						# Username mysql
$dbpassm  = 'PASSWORD';						# Password mysql
$dbuserw  = 'webchat';						# Username webchat
$dbpassw  = 'PASSWORD';						# Password webchat
$dbport   = 3306;						# Port (Default 3306)
$dbemail  = 'postmaster';					# E-Mail address for superuser


#####################
# Document Locations

$prefix      = getcwd();					# Prefix directory
$docaup      = "$prefix/docs/aup.txt";				# Acceptable use policy
$docinvi     = "$prefix/docs/invitation.txt";			# Invatation when joining
$dochead     = "$prefix/docs/header.txt";			# Default header 
$docheadj    = "$prefix/docs/headerjava.txt";			# Jabascript header 
$docfoot     = "$prefix/docs/footer.txt";			# Default footer
$docrealm    = "$prefix/docs/realminfo.txt";			# Default realm-info
$docbio      = "$prefix/docs/bio.txt";				# Default user bio
$docregist   = "$prefix/docs/register.txt";			# Registration text
$docjava     = "$prefix/docs/javascript.txt";			# Javascript text
$docconnect  = "$prefix/docs/connect.txt";			# Connection failure to realm notice

#####################
# General

my $Database;							# Database handle


#################################################################################
# Program Area
#################################################################################

	VarDisplay();						# Display environment variables
	$Database = DataConnectMysql();				# Connect to MySQL
	
	# Functions related directly to MySQL database
	print "\n\nDATABASE SPECIFICS\n";

	unless (DataVerifyUser($Database))			# Check for webchat user
	{ DataCreateUser($Database); }

	unless (DataVerifyDataAccess($Database))		# Data access to database
	{ DataCreateDataAccess($Database); }

	unless (DataVerifyDatabase($Database))			# Check if database exits
	{ DataCreateDatabase($Database); }

	$Database->disconnect;
	$Database = DataConnectWebchat();			# Connect to webchat


	# Functions related directly to webchat
	print "\n\nEXISTENCE OF TABLES\n";

	DataCreateTables($Database);				# Create all tables if needed

	print "\n\nCREATING SUPERUSER\n";
	unless (DataVerifySuperuser($Database))			# Create superuser account
	{ DataCreateSuperuser($Database); }

	print "\n\nCREATING DEFAULT DOCUMENTS\n";
	DocumentInsertAll($Database);

	$Database->disconnect;


#################################################################################
# Sub-Routines
#################################################################################

#####################
# CryptLenght
#
# Subroutine designed to discover the lenght of crypted text handed to it. 
# This is primarily to increase protability since some Perl distributions
# do not use the classic UNIX salted passwords of ten characters.

sub CryptLenght							# Salt, Word
{
	#####################
	# Data Members
	my $salt = shift;					# Salt used
	my $word = shift;					# Word applied


	#####################
	# Program Area
	$_ = crypt($word, $salt);				# Crypt password
	return tr/A-Za-z0-9.$\///;				# Return lenght of crypt
}


#####################
# DataConnect
#
# Subroutine designed to hide the code for database connection.

sub DataConnect							# MySQL or Webchat Datasource
{
	#####################
	# Data Members
	my $action = shift;					# Action to take	
	my $Database;						# Database handle

	#####################
	# Program Area

	# Connect to MySQL
	if ($action != 1)
	{
		$Database = DBI->connect("DBI:mysql:$dbdatam:$dbserver:$dbport", 
					$dbuserm, $dbpassm, {PrintError => 1})
			|| die ("Database Error \#$DBI::err :  $DBI::errstr"); 
	}

	# Connect to Webchat
	else
	{
		$Database = DBI->connect("DBI:mysql:$dbdataw:$dbserver:$dbport", 
					$dbuserm, $dbpassm,  {PrintError => 1})
			|| die ("MySQL Error \#$DBI::err :  $DBI::errstr"); 
	}

	return $Database;
}

# Alias for MySQL
sub DataConnectMysql
{ return DataConnect(0); }

# Alias for Webchat
sub DataConnectWebchat
{ return DataConnect(1); }


#####################
# DataCreate
#
# Creates needed fields, based on SQL received

sub DataCreate
{
	#####################
	# Data members
	my $Database = shift;					# Database handle

	my $sql = shift;					# SQL syntax


	#####################
	# Program Area
	$Database->do($sql);					# Create handle	
}

# User creation alias
sub DataCreateUser
{ 	
	# Print creation message
	print "\t........................................$dbuserw CREATED\n";

	DataCreate(shift, "INSERT INTO user (host, user, password) 
		VALUES('$dbserver', '$dbuserw', password('$dbpassw'))"); 
}

# Data creation alias
sub DataCreateDataAccess
{
	# Print creation message
	print "\t........................................Data Access CREATED\n";

	DataCreate(shift, "INSERT INTO db VALUES('$dbserver', '$dbdataw', '$dbuserw', 
		'Y', 'Y', 'Y', 'Y', 'N', 'N', 'N', 'N', 'N', 'N')");
}

# Database creation alias
sub DataCreateDatabase
{
	# Print creation message
	print "\t........................................$dbdataw CREATED\n";

	DataCreate(shift, "CREATE DATABASE $dbdataw");
}


#####################
# DataCreateTables
#
# Rather long sub-routine that carries with it all the needed information needed
# to create all tables in the Webchat database.

sub DataCreateTables
{
	#####################
	# Data members
	my $Database = shift;					# Database handle
	my @tables = qw(Puppeteer Puppet Realm RealmSupervisor Post PuppeteerRestriction PuppetInvitation PuppeteerRegistration WordRestriction WebchatText PuppetIgnore Audit);
	my $table;


	#####################
	# Program area

	# Run through list of tables
	foreach $table (@tables)
	{
		# Verify is non existent create
		unless (DataVerifyTables($Database, $table))
		{
			# Print creation message
			print "\t........................................$table CREATED\n";

			# Puppeteer
			# 
			# Primary login for all of associated puppets.  Contains basic
			# information about Puppeteer as to make contact if needed.
			# Also contrains preferences and rights.
			if ($table eq 'Puppeteer')
			{
				$Database->do("CREATE TABLE Puppeteer ( 
					PuppeteerLogin VARCHAR($PuppeteerLogin) PRIMARY KEY, 
					PuppeteerPassword CHAR($PuppeteerPassword) NOT NULL,
					PuppeteerName VARCHAR($PupeteerName) NOT NULL,
					PuppeteerEMail VARCHAR($PuppeteerEMail) NOT NULL,
					PuppeteerTimestamp INT,
					PuppeteerFormatting ENUM('complete','imageless','spartan') NOT NULL DEFAULT 'complete',
					PuppeteerChatPref ENUM('basic','framed','stream') NOT NULL DEFAULT 'basic',
					PuppeteerJavascript ENUM('yes','no') NOT NULL DEFAULT 'yes',
					PuppeteerAdminPrivs ENUM('yes','no') DEFAULT 'no',
					PuppeteerSuperPrivs ENUM('yes','no') DEFAULT 'no'
				)");
			}

			# Puppet
			#
			# Identities by which the puppeteer is known.  Each can have a 
			# unique set of attributes for which to be used.
			elsif ($table eq 'Puppet')
			{
				$Database->do("CREATE TABLE Puppet ( 
					PuppetName VARCHAR($PuppetName) PRIMARY KEY,
					PuppeteerLogin VARCHAR($PuppeteerLogin) NOT NULL,
					PuppetTag BLOB,
					PuppetPic VARCHAR($PuppetPic),
					PuppetPicHeight SMALLINT,
					PuppetPicWidth SMALLINT,
					PuppetTitle BLOB,
					PuppetBio BLOB,
					PuppetTextColour VARCHAR($PuppetTextColour) DEFAULT 'black',	
					PuppetTextSize TINYINT DEFAULT '0'
				)");
			}
			
			# Realm
			#
			# Can be consider worlds in which a puppet can be played in
			# All realms have a level of customization except for Private
			# Realsm which adhere to defaults.
			elsif ($table eq 'Realm')
			{
				$Database->do("CREATE TABLE Realm ( 
					RealmName VARCHAR($RealmName) PRIMARY KEY,
					RealmLanguage ENUM('relaxed','restricted') NOT NULL DEFAULT 'relaxed',
					RealmAccess ENUM('relaxed','restricted','list') NOT NULL DEFAULT 'relaxed',
					RealmGuests ENUM('yes','no','post') NOT NULL DEFAULT 'yes',
					RealmPublic ENUM('yes','no') NOT NULL DEFAULT 'yes',
					RealmTags ENUM('relaxed','restricted') NOT NULL DEFAULT 'relaxed',
					RealmInfo BLOB,
					RealmSidebar BLOB,	
					RealmHeader BLOB,
					RealmFooter BLOB,
					RealmImageHeight SMALLINT DEFAULT '0',
					RealmImageWidth SMALLINT DEFAULT '0',
					RealmTimestamp INT
				)");
			}

			# Realm Supervisor
			#
			# Realm supervisors have creative control over the Realm such
			# as colour and HTML as well as control over access.
			elsif ($table eq 'RealmSupervisor')
			{
				$Database->do("CREATE TABLE RealmSupervisor ( 
					RealmName VARCHAR($RealmName) NOT NULL,
					PuppeteerLogin VARCHAR($PuppeteerLogin) NOT NULL,
					UNIQUE PKRealmSupervisor (RealmName, PuppeteerLogin)
				)");
			}

			# Post
			#
			# A post is a message composed by a puppeteer from a puppet and
			# sent to other puppets within a Realm.  They do not migrate.
			elsif ($table eq 'Post')
			{
				$Database->do("CREATE TABLE Post ( 
					PostIDNumber INTEGER AUTO_INCREMENT PRIMARY KEY,
					RealmName VARCHAR($RealmName) NOT NULL,
					PuppetName VARCHAR($PuppetName) NOT NULL,
					PostTo VARCHAR($PuppetName) NOT NULL,
					PostTimestamp INT,
					PostFullFormat BLOB,
					PostImagelessFormat BLOB,
					PostPartialFormat BLOB
				)");
			}

			# Word Resriction
			#
			# Optional in realms but allows a supervisor to restrict certain
			# words used in the Realm itself.  Normally swears.
			elsif ($table eq 'WordRestriction')
			{
				$Database->do("CREATE TABLE WordRestriction ( 
					WordSpelling VARCHAR($WordSpelling) NOT NULL, 	
					RealmName VARCHAR($RealmName) NOT NULL,
					UNIQUE PKWordRestriction (WordSpelling, RealmName)
				)");
			}

			# Puppeteer Restirctions
			#
			# This will prevents a puppeteers from entering a realm.   This
			# is not permenant and will be revoked at a certain time.  For
			# lenght decided by admin.
			elsif ($table eq 'PuppeteerRestriction')
			{
				$Database->do("CREATE TABLE PuppeteerRestriction ( 
					PuppeteerLogin VARCHAR($PuppeteerLogin) NOT NULL,
					RealmName VARCHAR($RealmName) NOT NULL,
					RestrictionTimestamp INT NOT NULL,
					UNIQUE PKPuppeteerRestriction (PuppeteerLogin, RealmName)
				)");
			}

			# Puppet Invitation
			#
			# Similiar to puppeteer restriction, except that it restricts who
			# can enter.  In this case puppets for tighter control.
			elsif ($table eq 'PuppetInvitation')
			{
				$Database->do("CREATE TABLE PuppetInvitation ( 
					PuppetName VARCHAR($PuppetName) NOT NULL,
					RealmName VARCHAR($RealmName) NOT NULL,
					UNIQUE PKPuppetInvitation (PuppetName, RealmName)
				)");
			}

			# Puppet Ignore
			#
			# Applied if a puppeteer wishes to no longer receive posts by
			# a certain puppet.
			elsif ($table eq 'PuppetIgnore')
			{
				$Database->do("CREATE TABLE PuppetIgnore ( 
					PuppeteerLogin VARCHAR($PuppeteerLogin) NOT NULL,
					PuppetName VARCHAR($PuppetName) NOT NULL,
					UNIQUE PKPuppetIgnore (PuppeteerLogin, PuppetName)
				)");
			}

			# Puppeteer Registration
			#
			# Denormorlized due to it's lack of use compared to the 
			# normal fields.  Used in registration, failure to 
			# register results in puppeteer being removed.
			elsif ($table eq 'PuppeteerRegistration')
			{
				$Database->do("CREATE TABLE PuppeteerRegistration ( 
					PuppeteerLogin VARCHAR($PuppeteerLogin) PRIMARY KEY,
					PuppeteerConfCode VARCHAR($PuppeteerConfCode) NOT NULL,
					PuppeteerTimeStamp INT NOT NULL
				)");
			}

			# Webcat Texts
			#
			# Contains various documents used in WebChat.  
			elsif ($table eq 'WebchatText')
			{
				$Database->do("CREATE TABLE WebchatText ( 
					WebchatTextName	VARCHAR($WebchatTextName) PRIMARY KEY,
					WebchatTextContent BLOB
				)");
			}

			# Audit
			#
			# Auditing table for supervisor actions
			elsif ($table eq 'Audit')
			{
				$Database->do("CREATE TABLE Audit ( 
					AuditNumber INT NOT NULL AUTO_INCREMENT,
					PuppeteerLogin VARCHAR($PuppeteerLogin) NOT NULL,
					AuditAction VARCHAR(200),
					AuditTimestamp DATETIME,
					UNIQUE PKAudit (AuditNumber, PuppeteerLogin)
				)");
			}
		}
	}
}


#####################
# Superuser Create
#
# Creates the first adminstrator of the webchat scripts.  Will actually use the 
# The existing username and password my MySQL.  Username will remain the same
# password will be of course changeable.

sub DataCreateSuperuser
{
	#####################
	# Data Members
	my $Database = shift;					# Database handle

	my $salt;						# Encryption salt
	my $crypt;						# Crypted password

	my $String = new String::Random;


	#####################
	# Program Area

	$salt = $String->randpattern("ssssssss");		# Create random salt
	$crypt = crypt($dbpassm, $salt);			# Encrypt password

	print "\t........................................Superuser CREATED\n";

	# Insert into database superuser, with defaults and full supervisory rights
	$Database->do("INSERT INTO Puppeteer VALUES(?, ?, ?, ?, NULL, 'complete','basic', 'yes', 'yes', 'yes')", 
			{}, $dbuserm, $crypt, $dbuserm, $dbemail);
}


#####################
# DataVerify
#
# Used to verify existence of certain information based on received SQL statement
# and the word to check.

sub DataVerify							# Database, SQL statement, keyword 
{
	#####################
	# Data members
	my $Database = shift;					# Database handle
	my $Statement;						# SQL statement

	my $sql = shift;					# SQL syntax
	my $keyword = shift;					# Search word


	#####################
	# Program Area

	# Prepare SQL
	$Statement = $Database->prepare($sql);			
	$Statement->execute();

	print "\n\tSearching for $keyword:\n";
	
	# Run through list
	while ($_ = $Statement->fetchrow())
	{ 
		# Search for keyword
		if ($_ eq $keyword)
		{
			# Close statement and return true
			print "\t........................................FOUND\n";
			$Statement->finish();
			return 1;
		}
	}

	# Close statement and return false	
	print "\t........................................NOT FOUND\n";
	$Statement->finish();
	return 0;
}

# Alias user field
sub DataVerifyUser
{ return DataVerify(shift, "SELECT distinct(user) FROM user WHERE user='$dbuserw'", $dbuserw); }

# Alias for database access
sub DataVerifyDataAccess
{ return DataVerify(shift, "SELECT distinct(user) FROM db WHERE user='$dbuserw' AND db='$dbdataw'", $dbuserw); }

# Alias for document existence
sub DataVerifyDocument
{ return DataVerify(shift, "SELECT WebchatTextName FROM WebchatText", shift); }

# Alias for superuser find
sub DataVerifySuperuser
{ return DataVerify(shift, "SELECT PuppeteerLogin FROM Puppeteer WHERE PuppeteerLogin='$dbuserm'", $dbuserm); }

# Alias for databases
sub DataVerifyDatabase
{ return DataVerify(shift, 'SHOW DATABASES', $dbdataw); }

# Alias for tables
sub DataVerifyTables
{ return DataVerify(shift, 'SHOW TABLES', shift); }




#####################
# Document Insert
#
# This sub-routine is blessed with the responsibility of being used for 
# the very useful task of taking in text files, writing them to a scalar
# and taking that sclar moving it into a row into the Webchat database.

sub DocumentInsert						# Database handle, Filename, Row
{
	#####################
	# Data members
	my $Database = shift;					# Database handle
	my $filename = shift;					# Filename
	my $rowname = shift;					# Name of row to find

	my $line;						# Single line 
	my $field;						# Field of data


	#####################
	# Program area

	# Check for existence first
	unless (DataVerifyDocument($Database, $rowname))
	{
		# Verify file access
		if ((-f $filename) && (-R $filename))
		{
			# Create and open filehandle
			open(FILE, $filename) or Die("Unable to open $filename: $!");
			print "\t........................................$filename READING\n";
	
			# Read file and retreive lines
			while ($line = <FILE>)
			{ 
				# Insert lines into field as needed
				# As to remove errors
				if (defined($field))
				{ $field = "$field$line"; }

				# As to remove errors
				else
				{ $field = $line; }
			}
	
			# Insert into database
			$Database->do("INSERT INTO WebchatText VALUES(?,?)", {}, $rowname, $field);
		
			close(FILE);				# Close filehandle

			# Print pwetty message
			print "\t........................................$rowname CREATED\n";
		}
		else
		{ print "\t........................................$filename NOT FOUND\n"; };		
	}
}

# Alias for Accepteable Use Policy
sub DocumentInsertAUP
{ DocumentInsert(shift, $docaup, 'AcceptableUsePolicy'); }

# Alias for Invitation
sub DocumentInsertInvitation
{ DocumentInsert(shift, $docinvi, 'Invitation'); }

# Alias for HTML Header
sub DocumentInsertHeader
{ DocumentInsert(shift, $dochead, 'WebHeader'); }

# Alias for HTML Header
sub DocumentInsertHeaderJ
{ DocumentInsert(shift, $docheadj, 'WebHeaderJava'); }

# Alias for HTML Footer
sub DocumentInsertFooter
{ DocumentInsert(shift, $docfoot, 'WebFooter'); }

# Alias for Realm Information
sub DocumentInsertRealm
{ DocumentInsert(shift, $docrealm, 'InfoRealm'); }

# Alias for Puppet Bio
sub DocumentInsertBio
{ DocumentInsert(shift, $docbio, 'InfoBio'); }

# Alias for Registration
sub DocumentInsertRegistration
{ DocumentInsert(shift, $docregist, 'Registration'); }

# Alias for Accepted registrations
sub DocumentInsertAccepted
{ DocumentInsert(shift, $docaccept, 'Accepted'); }

# Alias for Connect failure
sub DocumentInsertConnect
{ DocumentInsert(shift, $docconnect, 'WebConnect'); }

# Alias for Javascript text
sub DocumentInsertJavascript
{ DocumentInsert(shift, $docjava, 'WebJavascript'); }

# Alias for all texts
sub DocumentInsertAll
{
	# Access all documents
	DocumentInsertAUP($_[0]);
	DocumentInsertInvitation($_[0]);
	DocumentInsertHeader($_[0]);
	DocumentInsertHeaderJ($_[0]);
	DocumentInsertFooter($_[0]);
	DocumentInsertSidebar($_[0]);
	DocumentInsertRealm($_[0]);
	DocumentInsertBio($_[0]);
	DocumentInsertRegistration($_[0]);
	DocumentInsertAccepted($_[0]);
	DocumentInsertConnect($_[0]);
	DocumentInsertJavascript($_[0]);
}


#####################
# VarDisplay
#
# Simple function to display variables are run-time for script Display are:
#	- Field sizes in tables
#	- MySQL connection information
#	- Document locations

sub VarDisplay
{
	#####################
	# Program Area
	print "FIELD SIZES (See schema for more information)\n";
	print "\tPuppeteerLogin    : $PuppeteerLogin\n";
	print "\tPuppeteerPassword : $PuppeteerPassword\n";
	print "\tPupeteername      : $PupeteerName\n";
	print "\tPuppeteerEMail    : $PuppeteerEMail\n";
	print "\tPuppetName        : $PuppetName\n";
	print "\tPuppetPic         : $PuppetPic\n";
	print "\tPuppetTextColour  : $PuppetTextColour\n";
	print "\tRealmName         : $RealmName\n";
	print "\tPostTo            : $PostTo\n";
	print "\tPuppeteerConfCode : $PuppeteerConfCode\n";
	print "\tWebchatTextName   : $WebchatTextName\n\n";

	print "MYSQL SETTINGS\n";
	print "\tMySQL server name : $dbserver\n";
	print "\tMySQL database    : $dbdatam\n";
	print "\tWebchat database  : $dbdataw\n";
	print "\tMySQL username    : $dbuserm\n";
	print "\tMySQL password    : $dbpassm\n";
	print "\tWebchat username  : $dbuserw\n";
	print "\tWebchat password  : $dbpassw\n";
	print "\tMySQL port number : $dbport\n\n";
	print "\tSuperuser E-Mail  : $dbemail\n\n";

	print "DOCUMENT LOCATIONS\n";
	print "\tCurrent directory : $prefix\n";
	print "\tAcceptable use    : $docaup\n";
	print "\tInvitation        : $docinvi\n";
	print "\tDefault header    : $dochead\n";
	print "\tDefault header J  : $docheadj\n";
	print "\tDefault footer    : $docfoot\n";
	print "\tDefault realminfo : $docrealm\n";
	print "\tDefault user BIO  : $docbio\n";
}
